/* Ace editor, element:textArea
    opts = {
        width:"100%",
        height:"auto",
        mode:"html" (default),
        disable:false,
    }
*/
    cuppaRP.aceEditor = function(elements, opts, aceOpts){
        this.opts = cuppaRP.mergeObjects([{width:"100%", height:"auto", mode:"html", disable:false}, opts]);
        this.aceOpts = cuppaRP.mergeObjects([{
                enableBasicAutocompletion:true, readOnly:false, showPrintMargin:true, displayIndentGuides:true,
                showLineNumbers:true, showGutter:true, enableSnippets: true, enableLiveAutocompletion: true, theme:"ace/theme/twilight",
                mode:"ace/mode/html"
            }, aceOpts]);
        if(elements == undefined) elements = ".ace";
        this.change = function(e, editor){ editor.textArea.value = editor.getValue(); };
        this.elements = cuppaRP.element(elements);
        this.elements.forEach(function(element){
            if(!element) return;
            if(element.aceProcessed) return;
            var aceOptions = cuppaRP.mergeObjects([this.aceOpts,   JSON.parse(cuppaRP.attribute(element, "ace-opts")) ], true);

            element.aceProcessed = true;
            cuppaRP.hide(element, {duration:0});
            var div = cuppaRP.newElement("<div></div>");
                div.element = element;
                cuppaRP.attribute(div, "id", "ace"+cuppaRP.unique());
                cuppaRP.css(div, {width:this.opts.width, height:this.opts.height, overflow:"hidden"});
                cuppaRP.after(div, element);
            var editor = window.ace.edit(div);
                editor.textArea = element;
                editor.getSession().setUseWorker(false);
                editor.getSession().setValue(element.value);
                editor.addEventListener('change', this.change);
                editor.setOptions(aceOptions);
                if(this.opts.height == "auto") editor.setOptions({ maxLines: Infinity });
        }.bind(this));
    };
/* Dropdown */
    cuppaRP.dropdown = function(element, buttonElement, opts){
        this.element = cuppaRP.element(element)[0];
        this.buttonElement = cuppaRP.element(buttonElement)[0];
        this.opts = cuppaRP.mergeObjects([{ scrollClose:true, stopProgation:false, mouseActionOpen:"click", toogle:true, mouseActionClose:"click", display:"block", open:null, close:null }, opts]);
        cuppaRP.css(this.element, {display:"none", opacity:0});
        // open Dropbox
            this.open = function(e){
                if(cuppaRP.hasClass(this.buttonElement, "cu-drop-open") && this.opts.toogle ){ this.close(); return; }
                cuppaRP.addClass(this.buttonElement, "cu-drop-open");
                cuppaRP.addClass(this.element, "cu-drop-open");
                window.TweenMax.to(this.element, 0.2, {alpha:1, display:this.opts.display});
                cuppaRP.off(window, this.opts.mouseActionClose, this.close);
                window.TweenMax.delayedCall(0.1, function(){cuppaRP.on(window, this.opts.mouseActionClose, this.close); }.bind(this));
                if(this.opts.scrollClose){ cuppaRP.off(window, "scroll", this.close); cuppaRP.on(window, "scroll", this.close); };
                if(this.opts.stopProgation) cuppaRP.on(element, "click", function(e){ e.stopPropagation(); });
                if(this.opts.open) this.opts.open();
            }.bind(this);
            if(this.opts.mouseActionOpen == "longPress"){
                cuppaRP.longPress(this.buttonElement, function(){
                    this.preventFirstClickClose = true;
                    this.open();
                }.bind(this));
            }else{
                cuppaRP.on(this.buttonElement, this.opts.mouseActionOpen, this.open);
            }
        // close Dropbox
            this.close = function(){
                if(this.preventFirstClickClose){
                    this.preventFirstClickClose = false; return;
                }
                cuppaRP.removeClass(this.buttonElement, "cu-drop-open");
                cuppaRP.off(window, this.opts.mouseActionClose, this.close);
                cuppaRP.off(window, "scroll", this.close);
                window.TweenMax.killTweensOf(this.element);
                window.TweenMax.to(this.element, 0.2, {alpha:0, display:"none"});
                if(this.opts.close) this.opts.close();
            }.bind(this);

        this.element.dropdown = this.element.buttonElement = this;
        return this;
    };
/* Serialize forms
    cuppaRP serialize, return the input values on a Object, name:value
*/
    cuppaRP.serialize = function(element, json_encode, base64_encode){
        if(json_encode == undefined) json_encode = false;
        if(base64_encode == undefined) base64_encode = true;
        element = cuppaRP.element(element,{query:true})[0];
        var formItems = element.querySelectorAll("input, textarea, select");
        var object = {}
        for(var i = 0; i < formItems.length; i++){
            var field = formItems[i];
            var name = field.getAttribute("name");
            if(name){
                if(field.getAttribute("type") == "checkbox" ){
                    object[name] = field.checked;
                }else if(field.getAttribute("type") == "radio"){
                    object[name] =  field.value;
                }else if(cuppaRP.elementType(field) == "select" && field.hasAttribute("multiple") ){
                    object[name] =  cuppaRP.selectedValues(field);
                }else{
                    object[name] = field.value;
                }
                if(typeof object[name] === "string") object[name] = cuppaRP.trim(object[name]);
            };
        };
        if(json_encode && object) object = cuppaRP.jsonEncode(object, base64_encode);
        return object;
    };
/* Select get selected values */
    cuppaRP.selectedValues = function(element, opts){
        opts = cuppaRP.mergeObjects([{returnFirst:false, returnLast:false, returnText:false}, opts]);
        var values = Array.prototype.slice.call(element.querySelectorAll('option:checked'),0).map(function(v,i,a) {
            if(opts.returnText) return v.innerHTML;
            return v.value;
        });
        if(opts.returnFirst && values.length) return values[0];
        if(opts.returnLast && values.length) return values[values.length-1];
        return values;
    };
/* Input filter.
    Example: new cuppaRP.inputFilter(element, '0-9');
*/
    cuppaRP.inputFilter = function(element, values){
        this.values = values || "0-9";
        this.values = "["+values+"]";
        this.validate = function(event){
            var input = event.currentTarget;
            if (event.type =='keypress'){
                // 8 = backspace, 9 = tab, 13 = enter, 35 = end, 36 = home, 37 = left, 39 = right, 46 = delete
                var key = event.charCode ? event.charCode : event.keyCode ? event.keyCode : 0;
                if(key == 8 || key == 9 || key == 13 || key == 35 || key == 36|| key == 37 || key == 39 || key == 46){
                    // if charCode = key & keyCode = 0
                    // 35 = #, 36 = $, 37 = %, 39 = ', 46 = .
                    if (event.charCode == 0 && event.keyCode == key){
                        return true;
                    };
                };

                if((key == 99|| key == 120 ) && event.ctrlKey){  // copy / cut
                    return;
                }else if(key == 118 && event.ctrlKey){ // paste
                    var string = event.clipboardData.getData('text/plain');
                    var regex = new RegExp('^('+this.values+')+$');
                }else{
                    var string = String.fromCharCode(key);
                    var regex = new RegExp(this.values);
                }
            }else if (event.type =='paste'){
                var string = event.clipboardData.getData('text/plain');
                var regex = new RegExp('^('+this.values+')+$');
            }else{ return; };

            if(!regex.test(string)){
                event.preventDefault();
            }
            return true;
        }.bind(this);
        cuppaRP.on(element, "keypress", this.validate, "cuppaRPInputFilter");
        cuppaRP.on(element, "paste", this.validate, "cuppaRPInputFilter");
    };
    cuppaRP.inputFilterRemove = function(element){
        cuppaRP.off(element, "keypress", "", "cuppaRPInputFilter");
        //cuppaRP.on(element, "keyup", this.validate, "cuppaRPInputFilter");
        cuppaRP.off(element, "paste", "", "cuppaRPInputFilter");
    };
/* try to auto ajust timezoneOffset in dates */
    cuppaRP.dateTimezoneOffsetAuto = function(date, remove){
        if(remove) date.setMinutes( date.getMinutes() - date.getTimezoneOffset() );
        else date.setMinutes( date.getMinutes() + date.getTimezoneOffset() );
        return date;
    };
    cuppaRP.dateAddTimezone = function(date){
        return cuppaRP.dateTimezoneOffsetAuto(date);
    };
    cuppaRP.dateRemoveTimezone = function(date){
        return cuppaRP.dateTimezoneOffsetAuto(date, true);
    };
/* time to date
    string formats:  12:35:26 PM or  18:35:26
* */
    cuppaRP.timeToDate = function(value){
        if(!value) value = "00:00:00";
        value = value.toUpperCase();
        let hours = parseInt(value.substr(0,2));
        if(value.indexOf("AM") != -1 && hours == 12) hours = 0;
        if(value.indexOf("PM") != -1 && hours != 12){ hours = hours+12; }
        let minutes = parseInt(value.substr(3,2));
        let seconds = parseInt(value.substr(6,2));
        let date = new Date();
            date.setHours(hours);
            date.setMinutes(minutes);
            date.setSeconds(seconds);
        return date;
    }
// dateFormat
    cuppaRP.dateFormat = function(date, format){
        if (!format) format = "MM/DD/YYYY H:M:S";
        var hours = date.getHours();
        var minutes = date.getMinutes();
        var seconds = date.getSeconds();
        format = cuppaRP.replace(format,"mm", ("0"+(date.getMonth() + 1)).slice(-2));
        format = cuppaRP.replace(format,"yyyy", date.getFullYear());
        format = cuppaRP.replace(format,"dd", ("0"+date.getDate()).slice(-2));
        format = cuppaRP.replace(format,"I", (minutes < 10) ? "0"+minutes : minutes );
            format = cuppaRP.replace(format,"M", (minutes < 10) ? "0"+minutes : minutes );
        format = cuppaRP.replace(format,"S", (seconds < 10) ? "0"+seconds : seconds );
        if(format.indexOf("A") != -1){
            if(hours < 12){
                format = cuppaRP.replace(format,"A", "AM");
                if(hours == 0) hours = 12;
            }else{
                format = cuppaRP.replace(format,"A", "PM");
                if(hours != 12) hours = hours - 12;
            }
        }
        format = cuppaRP.replace(format,"H", (hours < 10) ? "0"+hours : hours);
        return format;
    };
/* Get Ages from date (YYYY/MM/DD) ) */
    cuppaRP.getAge = function(dateString) {
        var today = new Date();
        var birthDate = new Date(dateString);
        var age = today.getFullYear() - birthDate.getFullYear();
        var m = today.getMonth() - birthDate.getMonth();
        if (m < 0 || (m === 0 && today.getDate() < birthDate.getDate())) { age--; }
        return age;
    };
/* Get Diferent of dates (YYYY/MM/DD) */
    cuppaRP.dateDiff = function(date1, date2, opts){
        if(!opts) opts = {}
        if(opts.addCero == undefined) opts.addCero = false;
        if(typeof date1 =="string"){
            date1 = cuppaRP.replace(date1, "-", "/");
            date1 = new Date(date1);
        }
        if(date2 == undefined){
            date2 = new Date();
        }else if(typeof date2 =="string"){
            date2 = cuppaRP.replace(date2, "-", "/");
            date2 = new Date(date2);
        }
        var diff;
        if(date2.getTime() > date1.getTime()) diff = new Date(date2.getTime() - date1.getTime());
        else diff = new Date(date1.getTime() - date2.getTime());

        var data = {}
            data.date = diff;
            data.time = diff.getTime(); // miliseconds
            data.years = diff.getUTCFullYear() - 1970; if(data.years < 0 ) data.years = 0;
            data.months = diff.getUTCMonth(); if(data.months < 0 ) data.months = 0;
            data.days = parseInt((diff) / (1000 * 60 * 60 * 24));
            data.hours = parseInt(Math.abs(diff) / (1000 * 60 * 60) % 24);
            data.minutes =  parseInt(Math.abs(diff) / (1000 * 60) % 60);
            data.seconds = parseInt(Math.abs(diff) / (1000) % 60);

            data.years = cuppaRP.parseFloat(data.years);
            data.months = cuppaRP.parseFloat(data.months);
            data.days = cuppaRP.parseFloat(data.days);
            data.hours = cuppaRP.parseFloat(data.hours);
            data.minutes = cuppaRP.parseFloat(data.minutes);
            data.seconds = cuppaRP.parseFloat(data.seconds);
        if(opts.addCero){
            data.years = (data.years > 9) ? data.years : "0"+data.years;
            data.months = (data.months > 9) ? data.months : "0"+data.months;
            data.days = (data.days > 9) ? data.days : "0"+data.days;
            data.hours = (data.hours > 9) ? data.hours : "0"+data.hours;
            data.minutes = (data.minutes > 9) ? data.minutes : "0"+data.minutes;
            data.seconds = (data.seconds > 9) ? data.seconds : "0"+data.seconds;
        }
        return data;
    };
/* secondsToTime conver 150 > 00:02:30
* */
    cuppaRP.secondsToTime = function (theSeconds){
        var hours  = parseInt(theSeconds / 3600);
            hours  = hours > 9 ? hours  : "0" + hours;
        var minutes  = parseInt((theSeconds  - (hours * 3600)) / 60);
            minutes  = minutes > 9 ? minutes : "0" + minutes ;
        var seconds  = parseInt(theSeconds  - ((hours * 3600) + (minutes * 60)));
            seconds  = seconds > 9 ? seconds  : "0" + seconds;
        return hours + ":" + minutes + ":" + seconds;
    };
    cuppaRP.seconds2Time = function(theSeconds){ return cuppaRP.secondsToTime(theSeconds); };
/* month diff */
    cuppaRP.monthDiff = function(d1, d2){
        if(d2 == undefined) d2 = new Date();
        var months;
        months = (d2.getFullYear() - d1.getFullYear()) * 12;
        months -= d1.getMonth() + 1;
        months += d2.getMonth();
        return months <= 0 ? 0 : months;
    };
    cuppaRP.dateMonthDiff = function(d1, d2){ return cuppaRP.monthDiff(d1,d2); };
// enter_key
    cuppaRP.enterKey = function(element, callBack, data){
        element.addEventListener("keyup", function(e){
            if(e.which == 13) {
                cuppaRP.blur();
                callBack(data);
            };
        });
    };
    cuppaRP.keyEnter = function(element, callBack, data){ cuppaRP.enterKey(element, callBack, data); };

// backspace_key and delete_key
    cuppaRP.deleteKey = function(element, callBack, data){
        element.addEventListener("keyup", function(e){
            if(e.which == 8 || e.which == 46) {
                cuppaRP.blur();
                callBack(data);
            };
        });
    };
    cuppaRP.keyDelete = function(element, callBack, data){ cuppaRP.deleteKey(element, callBack, data); };

// exit key
    cuppaRP.exitKey = function(element, callBack, data){
        element.addEventListener("keyup", function(e){
            if(e.which == 27) {
                cuppaRP.blur();
                callBack(data);
            };
        });
    };
    cuppaRP.keyExit = function(element, callBack, data){ cuppaRP.exitKey(element, callBack, data); };
// long click press
    cuppaRP.longPress = function(element, callback, data, delay){
        element.timer = null;
        element.delay = delay || 300;
        element.callback = callback;
        element.data = data;
        element.addEventListener("mousedown", function(e){
            var item = e.currentTarget;
                item.timer = setTimeout(function(){
                    clearTimeout(item.timer)
                    if(item.callback) item.callback(item, item.data);
                }, item.delay);
        });

        element.addEventListener("mouseup", function(e){
            var item = e.currentTarget;
            clearTimeout(item.timer)

        });
    };

/* pagination */
    cuppaRP.pagination = function(opts){
        this.opts = { currentPage:1, totalPages:5, callback:null, append:null, update:null, selectPageLabel:"Select Page", pageLabel:"Page", nextLabe:"Next", lastLabel:"Last", backLabel:"Back", firstLabel:"First" }
        this.opts = cuppaRP.mergeObjects([this.opts, opts]);
        if(!this.opts.currentPage) this.opts.currentPage = 1;
        this.element = cuppaRP.newElement("<div class='cu-pagination'></div>");
        if(this.opts.totalPages <= 1){
            if(this.opts.append) this.opts.append.innerHTML = "";
            if(this.opts.update) this.opts.update.innerHTML = "";
            return;
        }
        // change
            this.change = function(e){
                var page = this.opts.currentPage;
                if(e.currentTarget.id == 'first') page = 1;
                else if(e.currentTarget.id == 'back') page--;
                else if(e.currentTarget.id == 'next') page++;
                else if(e.currentTarget.id == 'last') page = this.opts.totalPages;
                else if(e.currentTarget.id == "select") page = cuppaRP.toNumber(e.currentTarget.value);
                if(page && page != this.opts.currentPage && this.opts.callback) this.opts.callback(page);
            }.bind(this);
        // back / first
            if(this.opts.currentPage > 1){
                var first = cuppaRP.newElement("<div id='first' class='cu-paginator-first cu-paginator-button'>"+ this.opts.firstLabel +"</div>");
                var back = cuppaRP.newElement("<div id='back' class='cu-paginator-back cu-paginator-button'>"+ this.opts.backLabel +"</div>");
                cuppaRP.append(first, this.element);
                cuppaRP.append(back, this.element);
                cuppaRP.on([first, back], "click", this.change);
            }
        // Label
            var label = cuppaRP.newElement("<div class='cu-paginator-label'>"+this.opts.pageLabel+" "+this.opts.currentPage+"/"+this.opts.totalPages+"</div>");
            cuppaRP.append(label, this.element);
        // next / last
            if(this.opts.currentPage < this.opts.totalPages){
                var next = cuppaRP.newElement("<div id='next' class='cu-paginator-next cu-paginator-button'>"+ this.opts.nextLabe +"</div>");
                var last = cuppaRP.newElement("<div id='last' class='cu-paginator-last cu-paginator-button'>"+ this.opts.lastLabel +"</div>");
                cuppaRP.append(next, this.element);
                cuppaRP.append(last, this.element);
                cuppaRP.on([next, last], "click", this.change);
            }
        // select
            var select = cuppaRP.newElement("<select id='select' class='cu-paginator-select'></select>");
            var opt = cuppaRP.newElement("<option>"+this.opts.selectPageLabel+"</option>");
            cuppaRP.append(opt, select);
            for(var i = 1; i <= this.opts.totalPages; i++){
                if(i != this.opts.currentPage){
                    var opt = cuppaRP.newElement("<option value='"+i+"'>"+i+"</option>");
                    cuppaRP.append(opt, select);
                };
            };
            cuppaRP.append(select, this.element);
            cuppaRP.on(select, "change", this.change);
        // append
            if(this.opts.append) cuppaRP.append(this.element, this.opts.append);
            if(this.opts.update) cuppaRP.update(this.element, this.opts.update);
        return this;
    };
/* Upload file
    Require jquery_file_upload
    <link href="js/jquery_file_upload/css/jquery_file_upload.css" rel="stylesheet" type="text/css" />
    <script src="js/jquery/jquery.js"></script>
    <script src="js/jquery_file_upload/vendor/jquery.ui.widget.js"></script>
    <script src="js/jquery_file_upload/jquery.iframe-transport.js"></script>
    <script src="js/jquery_file_upload/jquery.fileupload.js"></script>
    Example: cuppaRP.fileUpload2('.div_element', opts);
*/
    cuppaRP.fileUpload = function(element, opts){
        if(opts == undefined) opts = {};
        if(!opts.php_path) opts.php_path = 'administrator/js/jquery_file_upload/server/php/';
        if(!opts.folder) opts.folder = "upload_files";
        if(!opts.unique_name) opts.unique_name = true;
        if(!opts.resize_width) opts.resize_width = "";
        if(!opts.resize_height) opts.resize_height = "";
        if(!opts.crop) opts.crop = "";
        if(!opts.compress) opts.compress = "";
        opts.element = element;
        opts.file = window.window.jQuery('<input class="cuppaRP-file-type" type="file" />').get(0);
        if(opts.accept) window.window.jQuery(opts.file).attr("accept",opts.accept)
        window.jQuery(opts.element).append(opts.file).css("overflow","hidden");
        //++ config
            var params = {}
            params.jqXHR = null;
            params.url = opts.php_path;
            params.formData = {path:opts.folder, unique_name:opts.unique_name, resize_width:opts.resize_width, resize_height:opts.resize_height, crop:opts.crop, compress:opts.compress};
            params.dataType = 'json';
            params.item_reference = opts.element;
            params.opts = opts;
            params.add = function (e, data) {
                var options = window.jQuery(this).data()[Object.keys(window.jQuery(this).data())[0]].options;
                options.jqXHR = data.submit();
            }
            params.send = function(e,  data){
                var name = data.files[0].name;
                var options = window.jQuery(this).data()[Object.keys(window.jQuery(this).data())[0]].options;
                options.file_name = name;
                if(options.opts.start) options.opts.start(options);
                if(options.opts.init) options.opts.init(options);
            }
            params.progressall = function(e, data){
                var options = window.jQuery(this).data()[Object.keys(window.jQuery(this).data())[0]].options;
                var progress = Math.round((data.loaded / data.total)*100);
                if(options.opts.progress) options.opts.progress( {options:options, progress:progress} );
            }
            params.done = function(e,data){
                var options = window.jQuery(this).data()[Object.keys(window.jQuery(this).data())[0]].options;
                var url = "media/"+options.opts.folder+"/"+data.result.files[0].name;
                if(options.opts.callback) options.opts.callback( {options:options, file:url, data:data.result.files[0] } );
                if(options.opts.complete) options.opts.complete( {options:options, file:url, data:data.result.files[0] } );
            }
            params.fail = function(e,data){
                var options = window.jQuery(this).data()[Object.keys(window.jQuery(this).data())[0]].options;
                try{ console.log(data.messages); }catch(err){}
                if(options.opts.error) options.opts.error(data);
            }
            window.jQuery(opts.file).fileupload(params);
        //--
    };
/* optionSelector
    This method receive a array of elements and add selected class to the element clicked similar to Input.Radio
        options.toggle = false|true
        options.multiple = false|true
        options.callback = options.change = function(item, data);
        options.default = element

*/
    cuppaRP.optionSelector = function(elements, options){
        this.options = cuppaRP.mergeObjects([{ toggle:true, multiple:false, callback:null, 'default':null }, options]);
        this.elements = cuppaRP.element(elements);
        this.change = function(element){
            if(element.currentTarget) element = element.currentTarget;
            var select = true;
            if(!this.options.multiple){
                if(this.options.toggle && cuppaRP.hasClass(element, "selected") ) select = false;
                cuppaRP.removeClass(this.elements, "selected");
                if(select) cuppaRP.addClass(element, "selected")
            }else{
                if(this.options.toggle && cuppaRP.hasClass(element, "selected") ) select = false;
                if(select) cuppaRP.addClass(element, "selected");
                else cuppaRP.removeClass(element, "selected");
            }
            if(this.options.callback) this.options.callback(element, this);
        }.bind(this);
        cuppaRP.on(this.elements, "click", this.change);
        if(this.options["default"]) this.change(this.options["default"])
    };
// validate
    cuppaRP.validateForm = function(form, opts){
        opts = cuppaRP.mergeObjects([{showError:true, errorMessage:"This field is required."}, opts]);
        let elements = cuppaRP.element("input,select,textarea", {parent:form});
        cuppaRP.removeClass(elements,"error");
        cuppaRP.remove(".form-error-meesage");
        let valid = true;
        for(var i = 0; i < elements.length; i++){
            let element = elements[i];
                element.errorElementId = "error-message-element"+i;
            if(!element.checkValidity()){
                cuppaRP.addClass(element,"error");
                valid = false;
                if(opts.showError){
                    var message = cuppaRP.attr(element,"error") || opts.errorMessage;
                    var errorElement = cuppaRP.newElement("<div class='form-error-message' id='"+element.errorElementId+"'>"+message+"</div>");
                    cuppaRP.after(errorElement, element);
                    cuppaRP.on(element, "input", function(e){
                        if(e.target.checkValidity()) cuppaRP.remove("#"+e.target.errorElementId);
                    });
                    cuppaRP.on(element, "change", function(e){
                        if(e.target.checkValidity()) cuppaRP.remove("#"+e.target.errorElementId);
                    });
                }
            }
        }
        return valid;
    };
/* toggle
    new cuppaRP.toggle(element, {callback:this.toggleColumn});
    element.selected = return true/false

    Structure:
    <div class="toggle">downloadFile
        <input type="hidden" name="enabled_rphone" value="0,1"> // if an input[type=hidden] is includded the toggle will automatically change the value inside that input
    </div>
* */
    cuppaRP.toggle = function(elements, opts){
        this.opts = cuppaRP.mergeObjects([{ callback:null, selected:undefined, initCallback:true }, opts]);
        this.elements = cuppaRP.element(elements);
        // onClick
            this.onClick = function(element, selected){
                if(element.currentTarget) element = element.currentTarget;

                if(element.selected) {
                    element.selected = false;
                    cuppaRP.removeClass(element, "selected");
                    if(element.input) element.input.value = element.selected;
                }else{
                    element.selected = true;
                    cuppaRP.addClass(element, "selected");
                    if(element.input) element.input.value = element.selected;
                }
                if(this.opts.callback) this.opts.callback( element, element.selected );
            }.bind(this);
        // init value
            cuppaRP.on(this.elements, "click", this.onClick);
            cuppaRP.addClass(this.elements, "cu-toggle");
            this.elements.forEach(function(element){
                element.input = element.querySelector("input");
                if(element.input) element.selected = parseInt(element.input.value);
                if(this.opts.selected != undefined) element.selected = this.opts.selected;
                if(element.selected == undefined) element.selected = cuppaRP.hasClass(element, "selected");
                if(element.selected) cuppaRP.addClass(element, "selected");
                if(this.opts.initCallback && this.opts.callback) this.opts.callback( element, element.selected );
            }.bind(this));
        return this;
    };
// download file
    cuppaRP.downloadFile = function(filePath, fileName, mimeType, opts){
        opts = cuppaRP.mergeObjects([{callback:null, preload:false, returnData:false, header:null, data:null, method:"GET", responseType:'blob'}, opts]);
        if(opts.preload){
            const xhttp = new XMLHttpRequest();
                xhttp.open(opts.method, filePath, true);
                if(opts.header){ for (var key in opts.header) { xhttp.setRequestHeader(key, opts.header[key]); }; };
                xhttp.responseType = opts.responseType;
                if(opts.data) xhttp.send(opts.data);
                else xhttp.send();
                xhttp.onload = function(){
                    if (xhttp.readyState === 4 && xhttp.status === 200) {
                        let data = new Blob([xhttp.response], {type: mimeType});
                        if(opts.callback && opts.returnData){ opts.callback(data); return; }
                        let link = document.createElement('a');
                            link.href = window.URL.createObjectURL(data);
                            link.download = fileName;
                        document.body.appendChild(link);
                            link.dispatchEvent(new MouseEvent('click', {bubbles: true, cancelable: true, view: window}));
                            link.remove();
                        window.URL.revokeObjectURL(link.href);
                        if(opts.callback){ opts.callback(link.href); }
                    }else{
                        if(opts.callback){ opts.callback(null); }
                    }
                };
                xhttp.onerror = function(){
                    log('onerror', xhttp)
                    if(opts.callback){ opts.callback(null); }
                }
        }else{
            let link = document.createElement('a');
                link.setAttribute("download", fileName);
                link.setAttribute("href", filePath);
                document.body.appendChild(link);
                link.dispatchEvent(new MouseEvent('click', {bubbles: true, cancelable: true, view: window}));
                link.remove();
                if(opts.callback) opts.callback(true);
        }
    };
